@@ -4,19 +4,21 @@
 from PyQt5.QtCore import QMimeDatabase
 
 from UM.Decorators import ascopy
+from typing import List
 
-##  Raised when a mime type can not be found.
+##  Raised when a MIME type can not be found.
 class MimeTypeNotFoundError(Exception):
     pass
 
-##  Simple value type class that encapsulates mime type data.
+##  Simple value type class that encapsulates MIME type data.
 class MimeType:
     ##  Constructor
     #
-    #   \param name The mime type name, like "text/plain".
-    #   \param comment A description of the mime type.
+    #   \param name The MIME type name, like "text/plain".
+    #   \param comment A description of the MIME type.
     #   \param suffixes A list of possible suffixes for the type.
-    #   \param preferred_suffix The preferred suffix for the type. Defaults to suffixes[0] if not specified.
+    #   \param preferred_suffix The preferred suffix for the type. Defaults to
+    #   ``suffixes[0]`` if not specified.
     def __init__(self, name, comment, suffixes, preferred_suffix = None):
         if name is None:
             raise ValueError("Name cannot be None")
@@ -38,37 +40,48 @@ class MimeType:
         else:
             self.__preferred_suffix = ""
 
-    ##  The name of the mime type.
+    ##  The name that identifies the MIME type.
     @property
     def name(self):
         return self.__name
 
-    ##  The comment of the mime type.
+    ##  The comment that describes of the MIME type.
     @property
     def comment(self):
         return self.__comment
 
-    ##  The list of suffixes for the mime type.
+    ##  The list of file name suffixes for the MIME type.
     @property
     def suffixes(self):
         return self.__suffixes
 
-    ##  The preferred suffix for the mime type.
+    ##  The preferred file name suffix for the MIME type.
     @property
     def preferredSuffix(self):
         return self.__preferred_suffix
 
+    ##  Gives a programmer-readable representation of the MIME type.
+    #
+    #   \return A string representing the MIME type.
     def __repr__(self):
         return "<MimeType name={0}>".format(self.__name)
 
+    ##  Indicates whether this MIME type is equal to another MIME type.
+    #
+    #   They are equal if the names match, since MIME types should have unique
+    #   names.
+    #
+    #   \return ``True`` if the two MIME types are equal, or ``False``
+    #   otherwise.
     def __eq__(self, other):
         return self.__name == other.name
 
-    ##  Strip the extension from a file name when it corresponds to one of the suffixes of this mime type.
+    ##  Strip the extension from a file name when it corresponds to one of the
+    #   suffixes of this MIME type.
     #
     #   \param file_name The file name to strip of extension.
-    #
-    #   \return file_name without extension or file_name when it does not match.
+    #   \return ``file_name`` without extension, or ``file_name`` when it does
+    #   not match.
     def stripExtension(self, file_name):
         suffixes = sorted(self.__suffixes.copy(), key = lambda i: len(i), reverse = True)
         for suffix in self.__suffixes:
@@ -79,11 +92,11 @@ class MimeType:
         return file_name
 
 
-    ##  Create a MimeType object from a QMimeType object.
+    ##  Create a ``MimeType`` object from a ``QMimeType`` object.
     #
-    #   \param qt_mime The QMimeType object to convert.
-    #
-    #   \return A new MimeType object with properties equal to the QMimeType object.
+    #   \param qt_mime The ``QMimeType`` object to convert.
+    #   \return A new ``MimeType`` object with properties equal to the
+    #   ``QMimeType`` object.
     @staticmethod
     def fromQMimeType(qt_mime):
         return MimeType(
@@ -93,22 +106,23 @@ class MimeType:
             preferred_suffix = qt_mime.preferredSuffix()
         )
 
-##  Handles lookup of mime types for files with support for custom mime types.
+##  Handles lookup of MIME types for files with support for custom MIME types.
 #
-#   This class wraps around QMimeDatabase and extends it with support for custom
-#   mime types defined at runtime.
+#   This class wraps around ``QMimeDatabase`` and extends it with support for
+#   custom MIME types defined at runtime.
 #
-#   \note Custom mime types are currently only detected based on extension.
+#   \note Custom MIME types are currently only detected based on extension.
 class MimeTypeDatabase:
-    ##  Get a mime type by name
+    ##  Get a MIME type by name.
     #
-    #   This will return a MimeType object corresponding to the specified name.
+    #   This will return a ``MimeType`` object corresponding to the specified
+    #   name.
     #
-    #   \param name The name of the mime type to return.
+    #   \param name The name of the MIME type to return.
+    #   \return A ``MimeType`` object corresponding to the specified name.
     #
-    #   \return A MimeType object corresponding to the specified name.
-    #
-    #   \exception MimeTypeNotFoundError Raised when the specified mime type cannot be found.
+    #   \exception MimeTypeNotFoundError Raised when the specified MIME type
+    #   cannot be found.
     @classmethod
     def getMimeType(cls, name):
         for mime in cls.__custom_mimetypes:
@@ -123,13 +137,13 @@ class MimeTypeDatabase:
 
     MimeTypeNotFoundError = MimeTypeNotFoundError
 
-    ##  Get a mime type for a specific file.
+    ##  Get a MIME type for a specific file.
     #
-    #   \param file_name The name of the file to get the mime type for.
-    #
-    #   \return A MimeType object that contains the detected mime type for the file.
-    #
-    #   \exception MimeTypeNotFoundError Raised when no mime type can be found for the specified file.
+    #   \param file_name The name of the file to get the MIME type for.
+    #   \return A MimeType object that contains the detected MIME type for the
+    #   file.
+    #   \exception MimeTypeNotFoundError Raised when no MIME type can be found
+    #   for the specified file.
     @classmethod
     def getMimeTypeForFile(cls, file_name):
         matches = []
@@ -155,14 +169,14 @@ class MimeTypeDatabase:
         if not mime.isDefault() and mime.isValid():
             return MimeType.fromQMimeType(mime)
 
-        raise MimeTypeNotFoundError("Could not find a valid mime type for {0}".format(file_name))
+        raise MimeTypeNotFoundError("Could not find a valid MIME type for {0}".format(file_name))
 
-    ##  Add a custom mime type that can be detected.
+    ##  Add a custom MIME type that can be detected.
     #
-    #   \param mime_type \type{MimeType} The custom mime type to add.
+    #   \param mime_type \type{MimeType} The custom MIME type to add.
     @classmethod
     def addMimeType(cls, mime_type):
         cls.__custom_mimetypes.append(mime_type)
 
     __system_database = QMimeDatabase()
-    __custom_mimetypes = []
+    __custom_mimetypes = [] # type: List[MimeType]
