@@ -1,7 +1,8 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the AGPLv3 or higher.
 
-from PyQt5.QtCore import QObject, pyqtProperty, pyqtSlot, Q_ENUMS, pyqtSignal
+from PyQt5.QtCore import QObject, pyqtProperty, Q_ENUMS, pyqtSignal
+from UM.FlameProfiler import pyqtSlot
 
 from datetime import timedelta
 import math
