@@ -4,46 +4,60 @@
 import os
 import re #For finding containers with asterisks in the constraints.
 import urllib #For ensuring container file names are proper file names
+import urllib.parse
 import pickle #For serializing/deserializing Python classes to binary files
+from typing import List, cast
+import collections
+import time
 
-from contextlib import contextmanager
-
+import UM.FlameProfiler
 from UM.PluginRegistry import PluginRegistry
 from UM.Resources import Resources, UnsupportedStorageTypeError
 from UM.MimeTypeDatabase import MimeType, MimeTypeDatabase
 from UM.Logger import Logger
 from UM.SaveFile import SaveFile
+from UM.Settings.Interfaces import ContainerInterface
 from UM.Signal import Signal, signalemitter
 from UM.LockFile import LockFile
 
 import UM.Dictionary
 
-from . import DefinitionContainer
-from . import InstanceContainer
-from . import ContainerStack
+MYPY = False
+if MYPY:
+    from UM.Application import Application
 
+from UM.Settings.DefinitionContainer import DefinitionContainer
+from UM.Settings.ContainerStack import ContainerStack
+from UM.Settings.InstanceContainer import InstanceContainer
+from UM.Settings.Interfaces import ContainerRegistryInterface
+from UM.Settings.Interfaces import DefinitionContainerInterface
+
+from . import ContainerQuery
 
 CONFIG_LOCK_FILENAME = "uranium.lock"
 
+# The maximum amount of query results we should cache
+MaxQueryCacheSize = 1000
 
 ##  Central class to manage all Setting containers.
 #
 #
 @signalemitter
-class ContainerRegistry:
+class ContainerRegistry(ContainerRegistryInterface):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
         self._emptyInstanceContainer = _EmptyInstanceContainer("empty")
 
-        self._containers = [self._emptyInstanceContainer]
+        self._containers = [self._emptyInstanceContainer]   # type: List[ContainerInterface]
         self._id_container_cache = {}
-        self._resource_types = [Resources.DefinitionContainers]
+        self._resource_types = [Resources.DefinitionContainers] # type: List[int]
+        self._query_cache = collections.OrderedDict() # This should really be an ordered set but that does not exist...
 
     containerAdded = Signal()
     containerRemoved = Signal()
 
-    def addResourceType(self, type):
+    def addResourceType(self, type: int) -> None:
         self._resource_types.append(type)
 
     ##  Find all DefinitionContainer objects matching certain criteria.
@@ -52,8 +66,8 @@ class ContainerRegistry:
     #   keys and values that need to match the metadata of the
     #   DefinitionContainer. An asterisk in the values can be used to denote a
     #   wildcard.
-    def findDefinitionContainers(self, **kwargs):
-        return self.findContainers(DefinitionContainer.DefinitionContainer, **kwargs)
+    def findDefinitionContainers(self, **kwargs) -> List[DefinitionContainerInterface]:
+        return cast(List[DefinitionContainerInterface], self.findContainers(DefinitionContainer, **kwargs))
 
     ##  Find all InstanceContainer objects matching certain criteria.
     #
@@ -61,16 +75,16 @@ class ContainerRegistry:
     #   keys and values that need to match the metadata of the
     #   InstanceContainer. An asterisk in the values can be used to denote a
     #   wildcard.
-    def findInstanceContainers(self, **kwargs):
-        return self.findContainers(InstanceContainer.InstanceContainer, **kwargs)
+    def findInstanceContainers(self, **kwargs) -> List[InstanceContainer]:
+        return cast(List[InstanceContainer], self.findContainers(InstanceContainer, **kwargs))
 
     ##  Find all ContainerStack objects matching certain criteria.
     #
     #   \param kwargs \type{dict} A dictionary of keyword arguments containing
     #   keys and values that need to match the metadata of the ContainerStack.
     #   An asterisk in the values can be used to denote a wildcard.
-    def findContainerStacks(self, **kwargs):
-        return self.findContainers(ContainerStack.ContainerStack, **kwargs)
+    def findContainerStacks(self, **kwargs) -> List[ContainerStack]:
+        return cast(List[ContainerStack], self.findContainers(ContainerStack, **kwargs))
 
     ##  Find all container objects matching certain criteria.
     #
@@ -82,11 +96,13 @@ class ContainerRegistry:
     #
     #   \return A list of containers matching the search criteria, or an empty
     #   list if nothing was found.
-    def findContainers(self, container_type = None, ignore_case = False, **kwargs):
-        containers = []
+    @UM.FlameProfiler.profile
+    def findContainers(self, container_type = None, *, ignore_case = False, **kwargs) -> List[ContainerInterface]:
+        # Create the query object
+        query = ContainerQuery.ContainerQuery(self, container_type, ignore_case = ignore_case, **kwargs)
 
-        if len(kwargs) == 1 and "id" in kwargs:
-            # If we are just searching for a single container by ID, look it up from the container cache
+        if query.isIdOnly():
+            # If we are just searching for a single container by ID, look it up from the ID-based cache
             container = self._id_container_cache.get(kwargs.get("id"))
             if container:
                 # Add an extra check to make sure the found container matches the requested container type.
@@ -96,100 +112,26 @@ class ContainerRegistry:
                 elif isinstance(container, container_type):
                     return [ container ]
 
-        for container in self._containers:
-            if container_type and not isinstance(container, container_type):
-                continue
+        if query in self._query_cache:
+            # If the exact same query is in the cache, we can re-use the query result
+            self._query_cache.move_to_end(query) # Query was used, so make sure to update its position
+            return self._query_cache[query].getResult()
 
-            matches_container = True
-            for key, value in kwargs.items():
-                try:
-                    if "*" in value:
-                        value = re.escape(value) #Escape for regex patterns.
-                        value = "^" + value.replace("\\*", ".*") + "$" #Instead of (now escaped) asterisks, match on any string. Also add anchors for a complete match.
-                        if ignore_case:
-                            value_pattern = re.compile(value, re.IGNORECASE)
-                        else:
-                            value_pattern = re.compile(value)
+        # Execute the query, then add it to the cache
+        query.execute()
+        self._query_cache[query] = query
 
-                        if key == "id":
-                            if not value_pattern.match(container.getId()):
-                                matches_container = False
-                            continue
-                        elif key == "name":
-                            if not value_pattern.match(container.getName()):
-                                matches_container = False
-                            continue
-                        elif key == "definition":
-                            try:
-                                if not value_pattern.match(container.getDefinition().getId()):
-                                    matches_container = False
-                                continue
-                            except AttributeError:  # Only instanceContainers have a get definition. We can ignore all others.
-                                pass
+        if len(self._query_cache) > MaxQueryCacheSize:
+            # Since we use an OrderedDict, we can use a simple FIFO scheme
+            # to discard queries. As long as we properly update queries
+            # that are being used, this results in the least used queries
+            # to be discarded.
+            self._query_cache.popitem(last = False)
 
-                        if not value_pattern.match(str(container.getMetaDataEntry(key))):
-                            matches_container = False
-                    elif not ignore_case:
-                        if key == "id":
-                            if value != container.getId():
-                                matches_container = False
-                            continue
-                        elif key == "name":
-                            if value != container.getName():
-                                matches_container = False
-                            continue
-                        elif key == "definition":
-                            try:
-                                if value != container.getDefinition().getId():
-                                    matches_container = False
-                                continue
-                            except AttributeError:  # Only instanceContainers have a get definition. We can ignore all others.
-                                pass
-
-                        if value != str(container.getMetaDataEntry(key)):
-                            matches_container = False
-                    else:
-                        if key == "id":
-                            if value.lower() != container.getId().lower():
-                                matches_container = False
-                            continue
-                        elif key == "name":
-                            if value.lower() != container.getName().lower():
-                                matches_container = False
-                            continue
-                        elif key == "definition":
-                            try:
-                                if value.lower() != container.getDefinition().getId().lower():
-                                    matches_container = False
-                                continue
-                            except AttributeError:  # Only instanceContainers have a get definition. We can ignore all others.
-                                pass
-
-                        if value.lower() != str(container.getMetaDataEntry(key)).lower():
-                            matches_container = False
-                except TypeError: #Value was not a string.
-                    if key == "id" or key == "name" or key == "definition":
-                        matches_container = False
-                        continue
-                    elif key == "read_only":
-                        try:
-                            if value != container.isReadOnly():
-                                matches_container = False
-                            continue
-                        except AttributeError:
-                            pass
-
-                    if value != container.getMetaDataEntry(key):
-                        matches_container = False
-                    continue
-
-            if matches_container:
-                containers.append(container)
-
-        return containers
+        return query.getResult()
 
     ##  This is a small convenience to make it easier to support complex structures in ContainerStacks.
-    def getEmptyInstanceContainer(self):
+    def getEmptyInstanceContainer(self) -> InstanceContainer:
         return self._emptyInstanceContainer
 
     ##  Load all available definition containers, instance containers and
@@ -197,7 +139,7 @@ class ContainerRegistry:
     #
     #   \note This method does not clear the internal list of containers. This means that any containers
     #   that were already added when the first call to this method happened will not be re-added.
-    def load(self):
+    def load(self) -> None:
         files = []
         for resource_type in self._resource_types:
             resources = Resources.getAllResourcesOfType(resource_type)
@@ -218,10 +160,10 @@ class ContainerRegistry:
 
                 type_priority = 2
 
-                if issubclass(container_type, DefinitionContainer.DefinitionContainer):
+                if issubclass(container_type, DefinitionContainer):
                     type_priority = 0
 
-                if issubclass(container_type, InstanceContainer.InstanceContainer):
+                if issubclass(container_type, InstanceContainer):
                     type_priority = 1
 
                 # Since we have the mime type and resource type here, process these two properties so we do not
@@ -233,7 +175,7 @@ class ContainerRegistry:
 
         # Sort the list of files by type_priority so we can ensure correct loading order.
         files = sorted(files, key = lambda i: i[0])
-
+        resource_start_time = time.time()
         for _, container_id, file_path, read_only, container_type in files:
             if container_id in self._id_container_cache:
                 Logger.log("c", "Found a container with a duplicate ID: %s", container_id)
@@ -241,7 +183,7 @@ class ContainerRegistry:
                 continue
 
             try:
-                if issubclass(container_type, DefinitionContainer.DefinitionContainer):
+                if issubclass(container_type, DefinitionContainer):
                     definition = self._loadCachedDefinition(container_id, file_path)
                     if definition:
                         self.addContainer(definition)
@@ -253,24 +195,32 @@ class ContainerRegistry:
                 new_container.setReadOnly(read_only)
                 new_container.setPath(file_path)
 
-                if issubclass(container_type, DefinitionContainer.DefinitionContainer):
+                if issubclass(container_type, DefinitionContainer):
                     self._saveCachedDefinition(new_container)
 
                 self.addContainer(new_container)
             except Exception as e:
                 Logger.logException("e", "Could not deserialize container %s", container_id)
+        Logger.log("d", "Loading data into container registry took %s seconds", time.time() - resource_start_time)
 
-    def addContainer(self, container):
+    @UM.FlameProfiler.profile
+    def addContainer(self, container: ContainerInterface) -> None:
         containers = self.findContainers(container_type = container.__class__, id = container.getId())
         if containers:
             Logger.log("w", "Container of type %s and id %s already added", repr(container.__class__), container.getId())
             return
 
+        if hasattr(container, "metaDataChanged"):
+            # Since queries are based on metadata, we need to make sure to clear the cache when a container's metadata changes.
+            container.metaDataChanged.connect(self._clearQueryCache)
+
         self._containers.append(container)
         self._id_container_cache[container.getId()] = container
+        self._clearQueryCache()
         self.containerAdded.emit(container)
 
-    def removeContainer(self, container_id):
+    @UM.FlameProfiler.profile
+    def removeContainer(self, container_id: str) -> None:
         containers = self.findContainers(None, id = container_id)
         if containers:
             container = containers[0]
@@ -278,6 +228,11 @@ class ContainerRegistry:
             self._containers.remove(container)
             del self._id_container_cache[container.getId()]
             self._deleteFiles(container)
+
+            if hasattr(container, "metaDataChanged"):
+                container.metaDataChanged.disconnect(self._clearQueryCache)
+
+            self._clearQueryCache()
             self.containerRemoved.emit(container)
 
             Logger.log("d", "Removed container %s", container.getId())
@@ -285,6 +240,7 @@ class ContainerRegistry:
         else:
             Logger.log("w", "Could not remove container with id %s, as no container with that ID is known", container_id)
 
+    @UM.FlameProfiler.profile
     def renameContainer(self, container_id, new_name, new_id = None):
         Logger.log("d", "Renaming container %s to %s", container_id, new_name)
         containers = self.findContainers(None, id = container_id)
@@ -308,9 +264,10 @@ class ContainerRegistry:
             container._id = new_id
             self._id_container_cache[container._id] = container # Keep cache up-to-date.
 
+        self._clearQueryCache()
         self.containerAdded.emit(container)
 
-    def saveAll(self):
+    def saveAll(self) -> None:
         for instance in self.findInstanceContainers():
             if not instance.isDirty():
                 continue
@@ -373,19 +330,19 @@ class ContainerRegistry:
     #   \param original The original name that may not be unique.
     #   \return A unique name that looks a lot like the original but may have
     #   a number behind it to make it unique.
-    def uniqueName(self, original):
+    def uniqueName(self, original: str) -> str:
         name = original.strip()
 
-        if not self.findContainers(id = name, ignore_case = True) and not self.findContainers(name = name):
-            return name # Nothing to do. Name is unique as it is.
-
-        num_check = re.compile("(.*?)\s*#\d$").match(name)
+        num_check = re.compile(r"(.*?)\s*#\d+$").match(name)
         if num_check: #There is a number in the name.
             name = num_check.group(1) #Filter out the number.
-        if name == "": #Wait, that deleted everything!
-            name = "Profile"
-        unique_name = name
 
+        if not name: #Wait, that deleted everything!
+            name = "Profile"
+        elif not self.findContainers(id = name, ignore_case = True) and not self.findContainers(name = name):
+            return original.strip()
+
+        unique_name = name
         i = 1
         while self.findContainers(id = unique_name, ignore_case = True) or self.findContainers(name = unique_name): #A container already has this name.
             i += 1 #Try next numbering.
@@ -498,6 +455,10 @@ class ContainerRegistry:
         with open(cache_path, "wb") as f:
             pickle.dump(definition, f)
 
+    # Clear the internal query cache
+    def _clearQueryCache(self, *args, **kwargs):
+        self._query_cache.clear()
+
     ##  Get the lock filename including full path
     #   Dependent on when you call this function, Resources.getConfigStoragePath may return different paths
     def getLockFilename(self):
@@ -527,29 +488,29 @@ class ContainerRegistry:
     def getApplication(cls):
         return cls.__application
 
-    __application = None
-    __instance = None
+    __application = None    # type: Application
+    __instance = None  # type: ContainerRegistry
 
     __container_types = {
-        "definition": DefinitionContainer.DefinitionContainer,
-        "instance": InstanceContainer.InstanceContainer,
-        "stack": ContainerStack.ContainerStack,
+        "definition": DefinitionContainer,
+        "instance": InstanceContainer,
+        "stack": ContainerStack,
     }
 
     __mime_type_map = {
-        "application/x-uranium-definitioncontainer": DefinitionContainer.DefinitionContainer,
-        "application/x-uranium-instancecontainer": InstanceContainer.InstanceContainer,
-        "application/x-uranium-containerstack": ContainerStack.ContainerStack,
-        "application/x-uranium-extruderstack": ContainerStack.ContainerStack,
+        "application/x-uranium-definitioncontainer": DefinitionContainer,
+        "application/x-uranium-instancecontainer": InstanceContainer,
+        "application/x-uranium-containerstack": ContainerStack,
+        "application/x-uranium-extruderstack": ContainerStack
     }
 
 PluginRegistry.addType("settings_container", ContainerRegistry.addContainerType)
 
-class _EmptyInstanceContainer(InstanceContainer.InstanceContainer):
-    def isDirty(self):
+class _EmptyInstanceContainer(InstanceContainer):
+    def isDirty(self) -> bool:
         return False
 
-    def isReadOnly(self):
+    def isReadOnly(self) -> bool:
         return True
 
     def getProperty(self, key, property_name):
@@ -559,5 +520,6 @@ class _EmptyInstanceContainer(InstanceContainer.InstanceContainer):
         Logger.log("e", "Setting property %s of container %s which should remain empty", key, self.getName())
         return
 
-    def serialize(self):
+    def serialize(self) -> str:
         return "[general]\n version = 2\n name = empty\n definition = fdmprinter\n"
+
