@@ -8,6 +8,9 @@ from UM.Logger import Logger
 from UM.Math.Vector import Vector
 
 from UM.View.GL.OpenGL import OpenGL
+from UM.View.GL.OpenGLContext import OpenGLContext
+
+from PyQt5.QtGui import QOpenGLVertexArrayObject
 
 vertexBufferProperty = "__gl_vertex_buffer"
 indexBufferProperty = "__gl_index_buffer"
@@ -20,6 +23,12 @@ indexBufferProperty = "__gl_index_buffer"
 #   individual objects. This means that for example the ShaderProgram used is
 #   only bound once, at the start of rendering. There are a few values, like
 #   the model-view-projection matrix that are updated for each object.
+#
+#   Currently RenderBatch objects are created each frame including the
+#   VertexArrayObject (VAO). This is done to greatly simplify managing
+#   RenderBatch-changes. Whenever (sets of) RenderBatches are managed throughout
+#   the lifetime of a session, crossing multiple frames, the usage of VAO's can
+#   improve performance by reusing them.
 class RenderBatch():
     ##  The type of render batch.
     #
@@ -192,6 +201,16 @@ class RenderBatch():
             light_0_position = camera.getWorldPosition() + Vector(0, 50, 0)
         )
 
+        # The VertexArrayObject (VAO) works like a VCR, recording buffer activities in the GPU.
+        # When the same buffers are used elsewhere, one can bind this VertexArrayObject to
+        # the context instead of uploading all buffers again.
+        if OpenGLContext.properties["supportsVertexArrayObjects"]:
+            vao = QOpenGLVertexArrayObject()
+            vao.create()
+            if not vao.isCreated():
+                Logger.log("e", "VAO not created. Hell breaks loose")
+            vao.bind()
+
         for item in self._items:
             self._renderItem(item)
 
@@ -227,7 +246,13 @@ class RenderBatch():
         vertex_buffer = OpenGL.getInstance().createVertexBuffer(mesh)
         vertex_buffer.bind()
 
-        index_buffer = OpenGL.getInstance().createIndexBuffer(mesh)
+        if self._render_range is None:
+            index_buffer = OpenGL.getInstance().createIndexBuffer(mesh)
+        else:
+            # glDrawRangeElements does not work as expected and did not get the indices field working..
+            # Now we're just uploading a clipped part of the array and the start index always becomes 0.
+            index_buffer = OpenGL.getInstance().createIndexBuffer(
+                mesh, force_recreate = True, index_start = self._render_range[0], index_stop = self._render_range[1])
         if index_buffer is not None:
             index_buffer.bind()
 
@@ -246,6 +271,21 @@ class RenderBatch():
             self._shader.enableAttribute("a_uvs", "vector2f", offset)
             offset += mesh.getVertexCount() * 2 * 4
 
+        for attribute_name in mesh.attributeNames():
+            attribute = mesh.getAttribute(attribute_name)
+            self._shader.enableAttribute(attribute["opengl_name"], attribute["opengl_type"], offset)
+            if attribute["opengl_type"] == "vector2f":
+                offset += mesh.getVertexCount() * 2 * 4
+            elif attribute["opengl_type"] == "vector4f":
+                offset += mesh.getVertexCount() * 4 * 4
+            elif attribute["opengl_type"] == "int":
+                offset += mesh.getVertexCount() * 4
+            elif attribute["opengl_type"] == "float":
+                offset += mesh.getVertexCount() * 4
+            else:
+                Logger.log("e", "Attribute with name [%s] uses non implemented type [%s]." % (attribute["opengl_name"], attribute["opengl_type"]))
+                self._shader.disableAttribute(attribute["opengl_name"])
+
         if mesh.hasIndices():
             if self._render_range is None:
                 if self._render_mode == self.RenderMode.Triangles:
@@ -256,7 +296,7 @@ class RenderBatch():
                 if self._render_mode == self.RenderMode.Triangles:
                     self._gl.glDrawRangeElements(self._render_mode, self._render_range[0], self._render_range[1], self._render_range[1] - self._render_range[0], self._gl.GL_UNSIGNED_INT, None)
                 else:
-                    self._gl.glDrawRangeElements(self._render_mode, self._render_range[0], self._render_range[1], self._render_range[1] - self._render_range[0], self._gl.GL_UNSIGNED_INT, None)
+                    self._gl.glDrawElements(self._render_mode, self._render_range[1] - self._render_range[0], self._gl.GL_UNSIGNED_INT, None)
         else:
             self._gl.glDrawArrays(self._render_mode, 0, mesh.getVertexCount())
 
@@ -264,6 +304,9 @@ class RenderBatch():
         self._shader.disableAttribute("a_normal")
         self._shader.disableAttribute("a_color")
         self._shader.disableAttribute("a_uvs")
+        for attribute_name in mesh.attributeNames():
+            attribute = mesh.getAttribute(attribute_name)
+            self._shader.disableAttribute(attribute.get("opengl_name"))
         vertex_buffer.release()
 
         if index_buffer is not None:
